Package: scDDboost
Type: Package
Title: A compositional model to assess expression changes from
        single-cell rna-seq data
Version: 1.13.0
Date: 2018-10-31
Authors@R: c(person("Xiuyu","Ma",email="watsonforfun@gmail.com",role = c("cre", "aut")),person(given = "Michael A.",family = "Newton", email="newton@stat.wisc.edu",role = "ctb"))
Description: scDDboost is an R package to analyze changes in the distribution of single-cell expression data between two experimental conditions. Compared to other methods that assess differential expression, scDDboost benefits uniquely from information conveyed by the clustering of cells into cellular subtypes. Through a novel empirical Bayesian formulation it calculates gene-specific posterior probabilities that the marginal expression distribution is the same (or different) between the two conditions. The implementation in scDDboost treats gene-level expression data within each condition as a mixture of negative binomial distributions.
License: GPL (>= 2)
Imports: Rcpp (>= 0.12.11), RcppEigen (>= 0.3.2.9.0),EBSeq,
        BiocParallel, mclust, SingleCellExperiment, cluster, Oscope,
        SummarizedExperiment, stats, methods
biocViews: SingleCell, Software, Clustering, Sequencing,
        GeneExpression, DifferentialExpression, Bayesian
Depends: R (>= 4.2), ggplot2
LinkingTo: Rcpp, RcppEigen, BH
Suggests: knitr, rmarkdown, BiocStyle, testthat
SystemRequirements: c++14
Roxygen: list(wrap=FALSE)
RoxygenNote: 7.1.2
VignetteBuilder: knitr
BugReports: https://github.com/wiscstatman/scDDboost/issues
URL: https://github.com/wiscstatman/scDDboost
git_url: https://git.bioconductor.org/packages/scDDboost
git_branch: devel
git_last_commit: 009edb1
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: yes
Packaged: 2025-11-04 00:43:06 UTC; biocbuild
Author: Xiuyu Ma [cre, aut],
  Michael A. Newton [ctb]
Maintainer: Xiuyu Ma <watsonforfun@gmail.com>

useDynLib(scDDboost)
import(RcppEigen)
importFrom(BiocParallel,MulticoreParam,bplapply)
importFrom(SingleCellExperiment,colData)
importFrom(ggplot2,ggplot,geom_bar,facet_grid,coord_polar,xlab,ylab,theme,element_rect,element_line,geom_violin,geom_jitter)
importFrom(mclust,adjustedRandIndex)
importFrom(Oscope,CalcMV)
importFrom(cluster, pam)
importFrom(EBSeq,MedianNorm)
importFrom(Rcpp, evalCpp)
importFrom(stats,dist,cor,sd,nlminb,rgamma)
importFrom(SummarizedExperiment,assays)
importFrom("methods", "is")
export(pdd,calD,detK,getDD,getSizeofDD,pat,extractInfo)

<b> About </b>

scDDboost is an R package to analyze changes in the distribution of single-cell expression data between two experimental conditions.
Compared to other methods that assess differential expression, scDDboost benefits uniquely from information conveyed by the clustering of cells into cellular subtypes.  Through a novel empirical Bayesian formulation it calculates gene-specific posterior probabilities that 
the marginal expression distribution is the same (or different) between the two conditions.  The implementation in scDDboost treats gene-level expression data within each condition as a mixture of negative binomial distributions.  


<b> Installation </b>

To install the R package:
```R
# install.packages("devtools")
devtools::install_github("wiscstatman/scDDboost")
```
A tutorial and examples can be found at Rpackage/vignette/ 

<b> Paper </b>

Ma, X., Korthauer, K., Kendziorski, C., and Newton, M. A. (2019). A Compositional Model To Assess Expression Changes From Single-Cell RNA-Seq Data. 
The Annals of Applied Statistics 15, no. 2 (2021): 880-901.  DOI: 10.1214/20-AOAS1423.  Formerly, <a href="https://www.biorxiv.org/content/10.1101/655795v1.abstract"> bioRxiv 655795 </a>

#' extract count matrix from SingleCellExperiment object
#'
#'@param data SingleCellExperiment object
#'@return list of count matrix and condition vector
#'
#' @examples
#' data(sim_dat) 
#' dat <- extractInfo(sim_dat)

#' @export

extractInfo <- function(data){
    if(!is(data,"SingleCellExperiment")){
        stop("input data must be SingleCellExperiment Object")
    }
    data_counts <- assays(data)$count
    cd <- colData(data)$conditions
    res <- list()
    res[['count_matrix']] <- data_counts
    res[['condition']] <- cd
    return(res)
}

#' generating partition patterns
#'
#' @param K number of elements
#' @return all possible partition of K elements
#' @examples
#' pat(3)

#' @export


pat<-function(K){
    if(K %% 1 != 0){
        stop("input must be integer")
    }
    .Call('pat', K=K)
}

#' check refinement relation between two clusters
#'
#' @param x a cluster
#' @param y a cluster
#' @return whether x refines y
#' @export




isRef <- function(x,y) {
    if(length(x) != length(y)){
        stop("x, y must have equal lenth")
    }
    if(sum(x %% 1) != 0 || sum(y %% 1) != 0){
        stop("x, y must be integer vectors")
    }
    .Call("isref", X=x,Y=y)
}

#' MLE for random weighting parameter
#'
#' @param D, distance matrix of cells
#' @param reltol, tolerance of convergence
#' @return MLE of random weighting parameter
#' @export

rwMle <- function(D,reltol){
    ctrl <- list()
    ctrl$rel.tol <- reltol
    
    invD <- 1 / D
    n <- ncol(D)
    diag(invD) <- 0
    
    mm <- sum(invD) / (n * (n - 1))
    vv <- sum(invD * invD) / (n * (n - 1)) - mm^2
    
    fit3 <- nlminb( start=c(2,2), objective=LL, x=D, d0 = mm / vv, lower=c(0,0), control = ctrl)
    
    a0 <- fit3$par[1]
    a1 <- fit3$par[2]
    a <- a0 + a1
    
    return(a)
}

#' likelihood function for hyperparameters estimation
#'
#' @param param parameters to be determined by MLE
#' @param x distance matrix of cells
#' @param d0 rate parameter of prior of 1 / true distance 
#' @return return hyperparameteres a.
#' @export



LL <- function(param, x, d0){
    a0 <- param[1]   #shape for prior
    a1 <- param[2]  #shape for sampling model, consistent with dividing
    
    n <- length(x)
    
    nc <- ncol(x)
    
    I <- matrix(1,nc,nc)
    
    I <- I - diag(nc)
    
    C <- d0 + a1 * x
    
    res <- (n - nc) * (lgamma(a0 + a1) - lgamma(a0) - lgamma(a1) + a0 * log(d0) + a1 * log(a1)) + sum((a1 - 1) * log(x + diag(nc))) - sum((a0 + a1) * log(C) * I)
    
    return(-res)
}

#' determine the number of clusters
#'
#' @param D distance matrix
#' @param epi threshold for cutting off
#' @return number of clusters
#' @examples
#' data(sim_dat)
#' dat <- extractInfo(sim_dat)
#' data_counts <- dat$count_matrix
#' bp <- BiocParallel::MulticoreParam(4)
#' D_c <- calD(data_counts,bp)
#' detK(D_c)

#' @export

detK <- function(D, epi = 1)
{
     
    tmp <- vapply(2:9,function(x) clusHelper(D,x), c(1,1))
    intra <- tmp[1,]
    inter <- tmp[2,]
    s <- intra / inter
    
    ss <- s
    
    if(min(ss) < epi){
    K <- which(ss < epi)[1] + 1
        }else{
        K <- 9
        }
    return(K)
}

#' function to get intra and inter distance for clusters
#'
#' @param D distance matrix
#' @param i number of clusters
#' @return vector of intra and inter distance
#' @export
clusHelper <- function(D,i){
    clusRes <- pam(D,i,diss = TRUE)
    intra <- as.numeric(clusRes$objective[1])
    x <- clusRes$id.med
    inter <- mean(D[x,x])
    return(c(intra,inter))
}

#' generate random clusterings
#'
#' @param D distance matrix of cells
#' @param a paramter for weights
#' @param K number of subtypes
#' @return random generated clustering of cells

genRClus <- function(D,a,K){
    n <- ncol(D)
    e <- rgamma(n,shape= a / 2, rate= a )
    bar <- D/outer(e,e,"+")
    cstar <- pam(bar, k = K, diss = TRUE)$clustering
    return(cstar)
}

#' accelerated empirical bayesian
#'
#' @param data single cell expression matrix, row as genes column as cells
#' @param conditions partition of cells
#' @param gclus partition of genes
#' @param sf size factors
#' @param iter maximum iteration step of EM
#' @param hyper hyper parameters for beta distributions
#' @param PP pattern of partitions
#' @param stp1 step size of hyperparameter alpha (shared by all units) in one step EM
#' @param stp2 step size of hyperparameter beta (unit specific) in one step EM
#' @return posterior probability of mean expression pattern
#' @export

EBS <- function(data,conditions,gclus,sf,iter = 10,hyper,PP,stp1,stp2) {
    if(!is.matrix(data)){
        stop("data must be a numerical matrix")
    }
    if(length(conditions) != ncol(data)){
        stop("incorrect length of conditions")
    }
    if(length(gclus) != nrow(data)){
        stop("incorrect length of gene cluster")
    }
    if(length(sf) != ncol(data)){
        stop("incorrect length of size factors")
    }
    if(length(hyper) != nrow(data) + 1){
        stop("incorrect length of hyper parameters")
    }
    if(!is.matrix(PP)){
        stop("partition pattern must be a numerical matrix")
    }
    
    .Call('EBS', X=data,Y=conditions,Z=gclus,W=sf, iter=iter,hyper=hyper,part=PP,step1 = stp1, step2 = stp2)
}

#' calculate distance matrix
#'
#' @param data transcripts
#' @param bp bioc parallel parameter
#' @return distance matrix
#' @examples
#' data(sim_dat)
#' dat <- extractInfo(sim_dat)
#' data_counts <- dat$count_matrix
#' bp <- BiocParallel::MulticoreParam(4)
#' D_c <- calD(data_counts,bp)

#' @export



calD <- function(data,bp){
    nc <- ncol(data)
    nr <- nrow(data)
    
    geneMax <- apply(data,1,max)
    tmp <- which(geneMax > 0)
    cl <- gCl(data[tmp,], bp)
    
    geneK <- apply(cl,1,max)
    tmp <- which(geneK > 1)
    f_cl <- cl[tmp,]
    geneK <- geneK[tmp]
    
    
    ng <- nrow(f_cl)
    
    
    D_E <- as.matrix(dist(t(f_cl),method = "manhattan")) / nrow(f_cl)
    
    m_ <- max(D_E)
    
    D_E <- D_E / m_
    
    D_cor <- (1 - cor(data)) / 2
   
    w1 <- 1 / sd(D_E)
    
    w2 <- 1 / sd(D_cor)
    
    total <- w1 + w2
    
    w1 <- w1 / total
    
    w2 <- w2 / total
    
    D_c <- w2*D_cor + w1*D_E
    
    return(D_c)
}

context("pdd")
test_that("pdd gives correct result",{
    data(sim_dat)
    data_counts <- assays(sim_dat)$counts
    conditions <- colData(sim_dat)$conditions
    rownames(data_counts) <- seq_len(1000)
    bp <- BiocParallel::MulticoreParam(2)
    D_c <- calD(data_counts,bp)
    pdd <- pdd(data = data_counts,cd = conditions, bp, D = D_c, random = FALSE)
    expect_equal(as.numeric(pdd[101]),0.008143924,tolerance=1e-6)
})

context("calD")
test_that("pdd gives correct result",{
    data(sim_dat)
    data_counts <- assays(sim_dat)$counts
    conditions <- colData(sim_dat)$conditions
    rownames(data_counts) <- seq_len(1000)
    bp <- BiocParallel::MulticoreParam(2)
    D_c <- calD(data_counts,bp)
    expect_equal(as.numeric(D_c[100,101]),0.01811737,tolerance=1e-6)
})

#' calculate PDD when add random noise in distance matrix
#'
#' @param data normalized preprocessed transcripts
#' @param cd condition label
#' @param K number of subgroups
#' @param D distance matrix of cells
#' @param a shape param for weights
#' @param sz size factors 
#' @param hp hyper parameters for EBSeq
#' @param Posp parition patterns
#' @param iter max number of iterations for EM in EBSeq
#' @param REF refinement relation matrix
#' @param stp1 step size of hyperparameter alpha (shared by all units) in one step EM
#' @param stp2 step size of hyperparameter beta (unit specific) in one step EM
#' @return posterior probabilities under random distance matrix
#' @keywords internal
#' @export

pddRandom <- function(data, cd, K, D, a, sz, hp, Posp, iter, REF, stp1, stp2){

    cstar <- genRClus(D,a,K)
    gcl <- seq_len(nrow(data))
    tmp <- getZ1Z2(cstar,cd)
    z1 <- tmp[[1]]
    z2 <- tmp[[2]]
    res <- EBS(data,cstar,gcl,sz,iter,hp,Posp,stp1,stp2)
    DE <- res$DEpattern
    PDD <- pddAggregate(z1,z2,Posp,DE,K,REF)
    PDD
}

#' gene_level cluster
#'
#' @param data transcripts
#' @param bp bioc parallel parameter
#' @return return a matrix whose row represent gene specific cluster
#' @export

gCl <- function(data, bp){
    nr <- nrow(data)
    Phi_mdf <- rep(1, nr)
    bt <- rep(1, nr)
    tryCatch(
    {
        sz <- rep(1, ncol(data))
        MV<-CalcMV(data,Sizes = sz, Plot = FALSE)
        Phi_mdf<-MV$Phi_mdf
        Q_mdf<-MV$Q_mdf
        bt<-1/Q_mdf-1
    },
    error = function(w) {
     message("estimation of hyper parameter failed, try naively assigned parameters")
        }
    ,finally = {
    clus <- bplapply(seq_len(nr),function(i) mcp(data[i,],1,c(Phi_mdf[i],bt[i]))+1,BPPARAM = bp)
    cl <- matrix(unlist(clus),ncol = ncol(data), byrow=TRUE)
    return(cl)
    })
}

#' log likelihood of aggregated multinomial counts z given aggregated proportions t
#'
#' @param z counts of each group in one condition
#' @param pp a partition
#' @param alpha parameter of double dirichlet prior
#' @return log likelihood of aggregated multinomial counts z given aggregated proportions t
#' @export

##no beta
lpzgt <- function(z,pp,alpha)
{
    # log prob
    # z a vector of counts
    # pp a partition
    # p(z|t) assuming alpha_k=1;
    tt <- tapply(z,pp,sum) ## sufficient stats over blocks
    alpha_b <- tapply(alpha,pp,sum)
    nn <- table(pp)  ## counts block sizes
    res <- sum(lgamma(tt + 1)) - sum(lgamma(z + 1)) + sum(lgamma(alpha_b)) - sum(lgamma(alpha)) + sum(lgamma(z + alpha)) - sum(lgamma(tt + alpha_b))
    res
}


#' log likelihood of z1,z2 given t1,t2
#'
#' @param z1 counts of each group in condition 1
#' @param z2 counts of each group in condition 2
#' @param pp a partition
#' @param alpha1 parameter of double dirichlet prior
#' @param alpha2 parameter of double dirichlet prior
#' @return log likelihood of z1,z2 given t1,t2
#' @export

##beta equals sum over alpha1 and alpha2
lpt1t2 <- function(z1,z2,pp,alpha1,alpha2)
{
    K <- length(z1)
    n1 <- sum(z1); n2 <- sum(z2)
    t1 <- tapply(z1,pp,sum) ## sufficient stats over blocks
    t2 <- tapply(z2,pp,sum) ## sufficient stats over blocks
    beta <- alpha1 + alpha2
    beta_b <- tapply(beta,pp,sum)
    tmp1 <- lgamma(n1+1) + lgamma(n2+1) - sum(lgamma(t1+1)) - sum(lgamma(t2+1))
    tmp2 <- lgamma(sum(beta_b)) - sum( lgamma(beta_b) )
    tmp3 <- sum(lgamma(t1+t2+beta_b))-lgamma(n1+n2+sum(beta_b))
    res <- tmp1+tmp2+tmp3
    res
}

#' posterior of proportion change given mixture double dirichlet prior
#'
#' @param z1 counts of each group in condition 1
#' @param z2 counts of each group in condition 2
#' @param pat partition patterns
#' @param alpha1 parameter of double dirichlet prior
#' @param alpha2 parameter of double dirichlet prior
#' @return posterior of proportion change
#' @export

mdd <- function(z1, z2, pat, alpha1, alpha2){

    
    lpz <- apply(pat,1,function(x) lpzgt(z1,x,alpha1) + lpzgt(z2,x,alpha2) + lpt1t2(z1,z2,x,alpha1,alpha2))
    
    post <- exp(lpz-max(lpz))
    post <- post/sum(post)
    return(post)
}

#' index of DD genes under FDR control
#'
#' @param pDD probability of genes being DD
#' @param FDR fdr to be controlled
#' @return index of positive genes
#' @examples
#' p_dd <- c(0.01,0.99,0.7,0.5)
#' getDD(p_dd)

#' @export

getDD <- function(pDD, FDR=0.01)

{
    
    ee <- 1-pDD
    
    oe <- sort(ee)
    
    or <- order(ee)
    
    ff <- cumsum(oe)/seq_len(length(oe))
    
    return(or[which(ff < FDR)])
    
}

#' scDDboost
#'
#' simulated data for demonstration, data are mixture negative binomial distributed
#'
#' @docType data
#'
#' @usage data(sim_dat)
#'
#' @format An object of class \code{"list"}.
#'
#' @keywords datasets
#'

#' @examples
#' data(sim_dat)
"sim_dat"

#' calculate posterior probabilities of a gene to be differential distributed
#'
#' @param data normalized preprocessed transcripts
#' @param cd conditions label
#' @param bp bioc parallel parameter
#' @param D distance matrix of cells or cluster of cells or a given clustering
#' @param epi tol for change of validity score in determining number of clusters
#' @param random boolean indicator of whether randomzation has been been implemented on distance matrix
#' @param norm boolean indicator of whether the input expression data is normalized
#' @param Upper bound for hyper parameters optimization
#' @param nrandom number of random generated distance matrix
#' @param iter max number of iterations for EM
#' @param reltol relative tolerance for optim on weighting paramters
#' @param stp1 step size of hyperparameter alpha (shared by all units) in one step EM
#' @param stp2 step size of hyperparameter beta (unit specific) in one step EM
#' @param K number of subtypes, could be user specified or determined internally(set to 0)
#' @return posterior probabilities of a gene to be differential distributed

#' @examples
#' data(sim_dat)
#' dat <- extractInfo(sim_dat)
#' data_counts <- dat$count_matrix
#' cd <- dat$condition
#' bp <- BiocParallel::MulticoreParam(4)
#' D_c <- calD(data_counts,bp)
#' pDD <- pdd(data_counts,cd,bp,D_c)
#' @export


pdd <- function(data, cd, bp, D, random = TRUE, norm = TRUE, epi = 1, Upper = 1000, nrandom = 50, iter = 20,reltol = 1e-3, stp1 = 1e-6, stp2 = 1e-2, K = 0){
    
    G <- nrow(data)
    
    rname <- rownames(data)
    if(is.null(rname)){
        rname <- vapply(seq_len(G),function(x) paste0("gene",x),"string")
    }
    rs <- rowSums(data)
    zGene <- which(rs == 0)
    msg <- paste0(length(zGene), " genes are all zero counts, not being considered in DD analysis")
    message(msg)
    
    selected <- which(rs > 0)
    
    data <- data[selected,]
    
    gcl <- seq_len(nrow(data))
    
    
    if(norm)
    {
        if(is.matrix(D)){
            sz <- rep(1, ncol(D))
        }else if(is.numeric(D)){
            sz <- rep(1, length(D))
        }
    }else{
    sz <- tryCatch({MedianNorm(data)},error = function(e){
        message("sizeFactor calculation failed, try normalized data")
    })
    }
    alpha <- 0.4
    beta <- 2
    
    hp <- rep(beta, 1 + nrow(data))
    hp[1] <- alpha
    
    
    if(!random){
        if(is.matrix(D)){
            if(K == 0){
                K <- detK(D,epi)
            }
            msg <- paste0("estimated number of subtypes: ",K)
            message(msg)
            ccl <- pam(D, k = K, diss = TRUE)$clustering
        }
        else{
            if(is.numeric(D))
            {
                ccl <- D
                K <- max(ccl)
            }
            else
            {
               stop("input ccl must either be a dissimilarity measure or a partition for cells")
            }
            
        }
        
        
        Posp <- pat(K)[[1]]
        REF <- gRef(Posp)
        if(K >= 2){
            res <- EBS(data,ccl,gcl,sz,iter,hp,Posp,stp1,stp2)
            DE <- res$DEpattern
        }
        else if(K == 1){
            message("There is only one cluster, no postive")
            return(rep(0,nrow(data)))
        }
        tmp <- getZ1Z2(ccl,cd)
        z1 <- tmp[[1]]
        z2 <- tmp[[2]]
        PDD <- pddAggregate(z1,z2,Posp,DE,K,REF)
        res <- rep(0,G)
        res[selected] <- PDD
        return(res)
    }
    else{
        if(K == 0){
            K <- detK(D,epi)
        }
        msg <- paste0("estimated number of subtypes: ",K)
        message(msg)
        
        if(K == 1){
            message("There is only one cluster, no postive")
            return(rep(0,nrow(data)))
        }
        
        Posp <- pat(K)[[1]]
        REF <- gRef(Posp)
        
        # MLE for random weighting parameter
        a <- rwMle(D,reltol)
        
        result <- bplapply(seq_len(nrandom), function(i) {pddRandom(data, cd, K, D, a, sz, hp, Posp, iter, REF,stp1,stp2)}, BPPARAM = bp)
        
        
        boot <- matrix(unlist(result),ncol = nrandom, byrow=FALSE)
        
        PDD <- rowSums(boot) / nrandom
        res <- rep(0,G)
        res[selected] <- PDD
        
        names(res) <- rname
        
        return (res)
    }
    
}

#' function to get counts of cluster sizes at two conditions
#'
#' @param ccl clustering label
#' @param cd condition label
#' @return return list of counts
#' @export

getZ1Z2 <- function(ccl,cd){
    K <- max(ccl)
    n1 <- table(cd)[1]
    n <- length(ccl)
    tmp_z1<-table(ccl[seq_len(n1)])
    tmp_z2<-table(ccl[seq.int(n1 + 1,n)])
    z1 <- rep(0,K)
    z2 <- rep(0,K)
    z1[as.numeric(names(tmp_z1))] <- as.numeric(tmp_z1)
    z2[as.numeric(names(tmp_z2))] <- as.numeric(tmp_z2)
    res <- list(z1,z2)
    res
}

#' function to aggregate intermediate results and get prob of DD
#'
#' @param z1 counts of cluster sizes in condition 1
#' @param z2 counts of cluster sizes in condition 2
#' @param Posp partition of cells
#' @param DE posterior probabilities of DE patterns
#' @param K number of clusters
#' @param REF reference matrix indicating relation of nested partitions
#' @return return vector of prob of DD
#' @export
pddAggregate <- function(z1,z2,Posp,DE,K,REF){
    alpha1 <- rep(1,K)
    alpha2 <- rep(1,K)
    post <- mdd(z1, z2, Posp, alpha1, alpha2)
    np <- nrow(Posp)
    modified_p <- t(REF) %*% post
    PED <- DE%*%modified_p
    PDD <- 1 - PED
    PDD
}

#' generate reference matrix
#'
#' @param Posp possible partition of data
#' @return return a matrix indicate the refinement relation between different partitions.
#' @export


gRef <- function(Posp){
    .Call("g_ref",Posp = Posp)
}

## ------------------------------------------------------------------------------------
##          Internal function -- modal-cluster for poisson gamma model
## ------------------------------------------------------------------------------------

## x an integer vector
## y mass parameter
## z shape and scale parameters for gamma distribution
## return cluster of x

mcp <- function(x,y = 1,z = c(1,1)) {
    if(!is.vector(x)){
        stop("cluster object must be vector")
    }
    if(sum(x %% 1) != 0){
        stop("x must be vector of integers")
    }
    if(length(z) != 2){
        stop("incorrect number of hyper parameters")
    }
    fit<-.Call("MCP",X = x,MASS = y, PARAM=z)
    return(fit)
}

#' number of DD genes under FDR control
#'
#' @param pDD estimated probability of being DD
#' @param FDR fdr to be controlled
#' @return number of positive genes
#' @examples
#' p_dd <- c(0.1,0.99,1,0.05,0.05)
#' getSizeofDD(p_dd)
#' @export

getSizeofDD <- function(pDD, FDR=0.01)
{
    
    ee <- 1-pDD
    
    oe <- sort(ee)
    
    ff <- cumsum(oe)/seq_len(length(oe))
    
    return( sum( ff <= FDR ) )
    
}

## ----options, include=FALSE, echo=FALSE---------------------------------------
library(BiocStyle)
knitr::opts_chunk$set(warning=FALSE, error=FALSE, message=FALSE)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("scDDboost")

## ----echo=FALSE---------------------------------------------------------------
library(ggplot2)
df <- data.frame(
  group = as.factor(rep(c("subtype1", "subtype2", "subtype3"),2)),
proportion = c(1/3,1/3,1/3,1/6,1/2,1/3),
  label = as.factor(rep(c("condition1","condition2"),each = 3))
  )
bp<- ggplot(df, aes(x=proportion,y = "", fill=group))+
geom_bar(width = 2, stat = "identity") + facet_grid(cols = vars(label)) 
pie <- bp + coord_polar("x", start=0) + xlab("subtype proportions") + ylab("")
pie +
  theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))

## ----echo = F-----------------------------------------------------------------
x1 <- rnorm(33,10,2)
x2 <- rnorm(33,10,2)
x3 <- rnorm(33,20,2)

x4 <- rnorm(17,10,2)
x5 <- rnorm(50,10,2)
x6 <- rnorm(33,20,2)

df <- data.frame(
  group = c(rep(c("subtype1", "subtype2", "subtype3"),each = 33), c(rep("subtype1",17),rep("subtype2",50),rep("subtype3",33))),
  conditions = factor(c(rep(1,99),rep(2,100))),
  values = c(x1,x2,x3,x4,x5,x6)
  )
p <- ggplot(df,aes(x = conditions,y = values)) + geom_violin() + geom_jitter(aes(colour = group)) + xlab("conditions") + 
  ylab("gene expressions")
  

p + theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))

## ----echo = F-----------------------------------------------------------------
x1 <- rnorm(33,10,2)
x2 <- rnorm(33,20,2)
x3 <-  rnorm(33,30,2)

x4 <- rnorm(17,10,2)
x5 <- rnorm(50,20,2)
x6 <- rnorm(33,30,2)

df <- data.frame(
  group = c(rep(c("subtype1", "subtype2", "subtype3"),each = 33), c(rep("subtype1",17),rep("subtype2",50),rep("subtype3",33))),
  conditions = factor(c(rep(1,99),rep(2,100))),
  values = c(x1,x2,x3,x4,x5,x6)
  )
p <- ggplot(df,aes(x = conditions,y = values)) + geom_violin() + geom_jitter(aes(colour = group)) + xlab("conditions") + 
  ylab("gene expressions")
  

p + theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))






## -----------------------------------------------------------------------------
suppressMessages(library(scDDboost))

## -----------------------------------------------------------------------------
data(sim_dat)

## -----------------------------------------------------------------------------
data_counts <- SummarizedExperiment::assays(sim_dat)$counts
conditions <- SummarizedExperiment::colData(sim_dat)$conditions
rownames(data_counts) <- seq_len(1000)

##here we use 2 cores to compute the distance matrix
bp <- BiocParallel::MulticoreParam(2)
D_c <- calD(data_counts,bp)

ProbDD <- pdd(data = data_counts,cd = conditions, bp = bp, D = D_c)

## -----------------------------------------------------------------------------
## determine the number of subtypes
K <- detK(D_c)

## -----------------------------------------------------------------------------
EDD <- which(ProbDD > 0.95)

## -----------------------------------------------------------------------------
EDD <- getDD(ProbDD,0.05)

## -----------------------------------------------------------------------------
sessionInfo()

---
title: "scDDboost"
author: 
- name: Xiuyu Ma
  email:  watsonforfun@gmail.com
output: 
  BiocStyle::html_document:
  toc_float: true
vignette: >
  %\VignetteIndexEntry{scDDboost Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r options, include=FALSE, echo=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(warning=FALSE, error=FALSE, message=FALSE)
```

# Installation

The package can be installed from bioconductor
```{r,eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("scDDboost")
```
Issue can be reported at "https://github.com/wiscstatman/scDDboost/issues"


# Introduction
scDDboost scores evidence of a gene being differentially distributed(DD) across two conditions for single cell RNA-seq data. Higher resolution brings several chanllenges for analyzing the data, specifically, the distribution of gene expression tends to have high prevalence of zero and multi-modes.
To account for those characteristics and utilizing some biological intuition,
we view the expression values sampled from a pool of cells mixed by distinct cellular subtypes blind to condition label. Consequently, the distributional change can be fully determined by the
the change of subtype proportions. One tricky part is that not any change of proportions will lead to a distributional change. Given that some genes could be equivalent expressed across several subtypes, even the individual subytpe proportion may differ between conditions but as long as the aggregated proportions over those subtypes remain the same between conditions, it will not introduce different distribution.
For example


```{r,echo=FALSE}
library(ggplot2)
df <- data.frame(
  group = as.factor(rep(c("subtype1", "subtype2", "subtype3"),2)),
proportion = c(1/3,1/3,1/3,1/6,1/2,1/3),
  label = as.factor(rep(c("condition1","condition2"),each = 3))
  )
bp<- ggplot(df, aes(x=proportion,y = "", fill=group))+
geom_bar(width = 2, stat = "identity") + facet_grid(cols = vars(label)) 
pie <- bp + coord_polar("x", start=0) + xlab("subtype proportions") + ylab("")
pie +
  theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))
```

Proportions of subtypes 1 and 2 changed between the 2 conditions. 
The gene is not DD if subtype 1 and 2 have the same expression level

```{r,echo = F}
x1 <- rnorm(33,10,2)
x2 <- rnorm(33,10,2)
x3 <- rnorm(33,20,2)

x4 <- rnorm(17,10,2)
x5 <- rnorm(50,10,2)
x6 <- rnorm(33,20,2)

df <- data.frame(
  group = c(rep(c("subtype1", "subtype2", "subtype3"),each = 33), c(rep("subtype1",17),rep("subtype2",50),rep("subtype3",33))),
  conditions = factor(c(rep(1,99),rep(2,100))),
  values = c(x1,x2,x3,x4,x5,x6)
  )
p <- ggplot(df,aes(x = conditions,y = values)) + geom_violin() + geom_jitter(aes(colour = group)) + xlab("conditions") + 
  ylab("gene expressions")
  

p + theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))
```

For subtype 1 and 2 have different expression level, there is different distribution
```{r, echo = F}
x1 <- rnorm(33,10,2)
x2 <- rnorm(33,20,2)
x3 <-  rnorm(33,30,2)

x4 <- rnorm(17,10,2)
x5 <- rnorm(50,20,2)
x6 <- rnorm(33,30,2)

df <- data.frame(
  group = c(rep(c("subtype1", "subtype2", "subtype3"),each = 33), c(rep("subtype1",17),rep("subtype2",50),rep("subtype3",33))),
  conditions = factor(c(rep(1,99),rep(2,100))),
  values = c(x1,x2,x3,x4,x5,x6)
  )
p <- ggplot(df,aes(x = conditions,y = values)) + geom_violin() + geom_jitter(aes(colour = group)) + xlab("conditions") + 
  ylab("gene expressions")
  

p + theme(panel.background = element_rect(
    fill = 'white', colour = 'black'),
    panel.grid.minor.x = element_line(size = 0.5),
    panel.grid.minor.y = element_line(size = 0.5),
    panel.grid.major.x = element_line(size = 0.5),
    panel.grid.major.y = element_line(size = 0.5),
    panel.grid.major = element_line(colour = "grey"))





```




# Posterior probability of a gene being DD 


`pdd` is the core function developed to quantify the
posterior probabilities of DD for input genes. 
Let's look at an example,

```{r}
suppressMessages(library(scDDboost))
```
Next, we load the toy simulated example a \textit{SingleCellExperiment} object that we will use for
identifying and classifying DD genes.
```{r}
data(sim_dat)
```
Verify that this object is a member of the SingleCellExperiment class and that it contains 200
cells and 1000 genes. The colData slot (which contains a dataframe of metadata for the
cells) should have a column that contains the biological condition or grouping of interest. In
this example data, that variable is the `condition` variable. Note that the input gene set needs
to be a matrix of normalized counts.
We run the function `pdd` 
```{r}
data_counts <- SummarizedExperiment::assays(sim_dat)$counts
conditions <- SummarizedExperiment::colData(sim_dat)$conditions
rownames(data_counts) <- seq_len(1000)

##here we use 2 cores to compute the distance matrix
bp <- BiocParallel::MulticoreParam(2)
D_c <- calD(data_counts,bp)

ProbDD <- pdd(data = data_counts,cd = conditions, bp = bp, D = D_c)
```
There are 4 input parameters needed to be specified by user, the dataset, the condition label, number of cpu cores used for computation and a distance matrix
of cells. Other input parameters have default settings.

## clustering of cells

We provide a default method of getting the distance matrix, archived by `calD`, in general `pdd` accept all valid distance matrix. User can also input a cluster label rather than distance matrix for the argument `D`, but the random distancing mechanism which relies on distance matrix will be disabled and `random` should be set to false.  

For the number of sutypes, we provide a default function `detK`, which consider the smallest number of sutypes such that the ratio of difference within cluster between difference between clusters become smaller than a threshold (default setting is 1). 

If user have other ways to determine $K$, $K$ should be specified in `pdd`. 
```{r}
## determine the number of subtypes
K <- detK(D_c)
```


If we set threshold to be 5% then we have estimated DD genes 
```{r}
EDD <- which(ProbDD > 0.95)
```

Notice that, pdd is actually local false discovery rate, this is a conservative estimation of DD genes. 
We could gain further power, 
let index gene by $g = 1,2,...,G$ and let $p_g = P(DD_g | \text{data})$, $p_{(1)},...,p_{(G)}$ be ranked local false discovery rate from small to large. 
To control the false discovery rate at 5\%, our positive set is those genes with the $s^*$ smallest lFDR, where
$$s^* = \text{argmax}_s\{s,\frac{\Sigma_{i = 1}^s p_{(i)}}{s} \leq 0.05\}$$
```{r}
EDD <- getDD(ProbDD,0.05)
```

Function `getDD` extracts the estimated DD genes using the above transformation. 



# Session Information {-}

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdd.R
\name{getZ1Z2}
\alias{getZ1Z2}
\title{function to get counts of cluster sizes at two conditions}
\usage{
getZ1Z2(ccl, cd)
}
\arguments{
\item{ccl}{clustering label}

\item{cd}{condition label}
}
\value{
return list of counts
}
\description{
function to get counts of cluster sizes at two conditions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdd.R
\name{lpzgt}
\alias{lpzgt}
\title{log likelihood of aggregated multinomial counts z given aggregated proportions t}
\usage{
lpzgt(z, pp, alpha)
}
\arguments{
\item{z}{counts of each group in one condition}

\item{pp}{a partition}

\item{alpha}{parameter of double dirichlet prior}
}
\value{
log likelihood of aggregated multinomial counts z given aggregated proportions t
}
\description{
log likelihood of aggregated multinomial counts z given aggregated proportions t
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isRef.R
\name{isRef}
\alias{isRef}
\title{check refinement relation between two clusters}
\usage{
isRef(x, y)
}
\arguments{
\item{x}{a cluster}

\item{y}{a cluster}
}
\value{
whether x refines y
}
\description{
check refinement relation between two clusters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pat.R
\name{pat}
\alias{pat}
\title{generating partition patterns}
\usage{
pat(K)
}
\arguments{
\item{K}{number of elements}
}
\value{
all possible partition of K elements
}
\description{
generating partition patterns
}
\examples{
pat(3)
}

\name{scDDboost-package}
\alias{scDDboost-package}
\alias{scDDboost}
\docType{package}
\title{
\packageTitle{scDDboost}
}
\description{
\packageDescription{scDDboost}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{scDDboost}
\packageIndices{scDDboost}
Package used to score evidence of differential distribution in single-cell RNA-seq data
}
\author{
\packageAuthor{scDDboost}

Maintainer: \packageMaintainer{scDDboost}
}
\references{
https://projecteuclid.org/journals/annals-of-applied-statistics/volume-15/issue-2/A-compositional-model-to-assess-expression-changes-from-single-cell/10.1214/20-AOAS1423.short
}
\keyword{ Empirical Bayes, clustering, random weighting, local false discovery rate }
\seealso{
https://github.com/wiscstatman/scDDboost/blob/master/DESCRIPTION
}
\examples{
data(sim_dat)
dat = extractInfo(sim_dat)
data_counts = dat$count_matrix
cd = dat$condition
bp <- BiocParallel::MulticoreParam(4)
D_c = calD(data_counts,bp)
pDD = pdd(data_counts,cd,bp,D_c)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detK.R
\name{detK}
\alias{detK}
\title{determine the number of clusters}
\usage{
detK(D, epi = 1)
}
\arguments{
\item{D}{distance matrix}

\item{epi}{threshold for cutting off}
}
\value{
number of clusters
}
\description{
determine the number of clusters
}
\examples{
data(sim_dat)
dat <- extractInfo(sim_dat)
data_counts <- dat$count_matrix
bp <- BiocParallel::MulticoreParam(4)
D_c <- calD(data_counts,bp)
detK(D_c)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EBS.R
\name{EBS}
\alias{EBS}
\title{accelerated empirical bayesian}
\usage{
EBS(data, conditions, gclus, sf, iter = 10, hyper, PP, stp1, stp2)
}
\arguments{
\item{data}{single cell expression matrix, row as genes column as cells}

\item{conditions}{partition of cells}

\item{gclus}{partition of genes}

\item{sf}{size factors}

\item{iter}{maximum iteration step of EM}

\item{hyper}{hyper parameters for beta distributions}

\item{PP}{pattern of partitions}

\item{stp1}{step size of hyperparameter alpha (shared by all units) in one step EM}

\item{stp2}{step size of hyperparameter beta (unit specific) in one step EM}
}
\value{
posterior probability of mean expression pattern
}
\description{
accelerated empirical bayesian
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rwMle.R
\name{rwMle}
\alias{rwMle}
\title{MLE for random weighting parameter}
\usage{
rwMle(D, reltol)
}
\arguments{
\item{D, }{distance matrix of cells}

\item{reltol, }{tolerance of convergence}
}
\value{
MLE of random weighting parameter
}
\description{
MLE for random weighting parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gRef.R
\name{gRef}
\alias{gRef}
\title{generate reference matrix}
\usage{
gRef(Posp)
}
\arguments{
\item{Posp}{possible partition of data}
}
\value{
return a matrix indicate the refinement relation between different partitions.
}
\description{
generate reference matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSizeofDD.R
\name{getSizeofDD}
\alias{getSizeofDD}
\title{number of DD genes under FDR control}
\usage{
getSizeofDD(pDD, FDR = 0.01)
}
\arguments{
\item{pDD}{estimated probability of being DD}

\item{FDR}{fdr to be controlled}
}
\value{
number of positive genes
}
\description{
number of DD genes under FDR control
}
\examples{
p_dd <- c(0.1,0.99,1,0.05,0.05)
getSizeofDD(p_dd)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genRClus.R
\name{genRClus}
\alias{genRClus}
\title{generate random clusterings}
\usage{
genRClus(D, a, K)
}
\arguments{
\item{D}{distance matrix of cells}

\item{a}{paramter for weights}

\item{K}{number of subtypes}
}
\value{
random generated clustering of cells
}
\description{
generate random clusterings
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdd.R
\name{mdd}
\alias{mdd}
\title{posterior of proportion change given mixture double dirichlet prior}
\usage{
mdd(z1, z2, pat, alpha1, alpha2)
}
\arguments{
\item{z1}{counts of each group in condition 1}

\item{z2}{counts of each group in condition 2}

\item{pat}{partition patterns}

\item{alpha1}{parameter of double dirichlet prior}

\item{alpha2}{parameter of double dirichlet prior}
}
\value{
posterior of proportion change
}
\description{
posterior of proportion change given mixture double dirichlet prior
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDD.R
\name{getDD}
\alias{getDD}
\title{index of DD genes under FDR control}
\usage{
getDD(pDD, FDR = 0.01)
}
\arguments{
\item{pDD}{probability of genes being DD}

\item{FDR}{fdr to be controlled}
}
\value{
index of positive genes
}
\description{
index of DD genes under FDR control
}
\examples{
p_dd <- c(0.01,0.99,0.7,0.5)
getDD(p_dd)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detK.R
\name{clusHelper}
\alias{clusHelper}
\title{function to get intra and inter distance for clusters}
\usage{
clusHelper(D, i)
}
\arguments{
\item{D}{distance matrix}

\item{i}{number of clusters}
}
\value{
vector of intra and inter distance
}
\description{
function to get intra and inter distance for clusters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gCl.R
\name{gCl}
\alias{gCl}
\title{gene_level cluster}
\usage{
gCl(data, bp)
}
\arguments{
\item{data}{transcripts}

\item{bp}{bioc parallel parameter}
}
\value{
return a matrix whose row represent gene specific cluster
}
\description{
gene_level cluster
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calD.R
\name{calD}
\alias{calD}
\title{calculate distance matrix}
\usage{
calD(data, bp)
}
\arguments{
\item{data}{transcripts}

\item{bp}{bioc parallel parameter}
}
\value{
distance matrix
}
\description{
calculate distance matrix
}
\examples{
data(sim_dat)
dat <- extractInfo(sim_dat)
data_counts <- dat$count_matrix
bp <- BiocParallel::MulticoreParam(4)
D_c <- calD(data_counts,bp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdd.R
\name{pddAggregate}
\alias{pddAggregate}
\title{function to aggregate intermediate results and get prob of DD}
\usage{
pddAggregate(z1, z2, Posp, DE, K, REF)
}
\arguments{
\item{z1}{counts of cluster sizes in condition 1}

\item{z2}{counts of cluster sizes in condition 2}

\item{Posp}{partition of cells}

\item{DE}{posterior probabilities of DE patterns}

\item{K}{number of clusters}

\item{REF}{reference matrix indicating relation of nested partitions}
}
\value{
return vector of prob of DD
}
\description{
function to aggregate intermediate results and get prob of DD
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-data.R
\docType{data}
\name{sim_dat}
\alias{sim_dat}
\title{scDDboost}
\format{
An object of class \code{"list"}.
}
\usage{
data(sim_dat)
}
\description{
simulated data for demonstration, data are mixture negative binomial distributed
}
\examples{
data(sim_dat)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pddRandom.R
\name{pddRandom}
\alias{pddRandom}
\title{calculate PDD when add random noise in distance matrix}
\usage{
pddRandom(data, cd, K, D, a, sz, hp, Posp, iter, REF, stp1, stp2)
}
\arguments{
\item{data}{normalized preprocessed transcripts}

\item{cd}{condition label}

\item{K}{number of subgroups}

\item{D}{distance matrix of cells}

\item{a}{shape param for weights}

\item{sz}{size factors}

\item{hp}{hyper parameters for EBSeq}

\item{Posp}{parition patterns}

\item{iter}{max number of iterations for EM in EBSeq}

\item{REF}{refinement relation matrix}

\item{stp1}{step size of hyperparameter alpha (shared by all units) in one step EM}

\item{stp2}{step size of hyperparameter beta (unit specific) in one step EM}
}
\value{
posterior probabilities under random distance matrix
}
\description{
calculate PDD when add random noise in distance matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdd.R
\name{lpt1t2}
\alias{lpt1t2}
\title{log likelihood of z1,z2 given t1,t2}
\usage{
lpt1t2(z1, z2, pp, alpha1, alpha2)
}
\arguments{
\item{z1}{counts of each group in condition 1}

\item{z2}{counts of each group in condition 2}

\item{pp}{a partition}

\item{alpha1}{parameter of double dirichlet prior}

\item{alpha2}{parameter of double dirichlet prior}
}
\value{
log likelihood of z1,z2 given t1,t2
}
\description{
log likelihood of z1,z2 given t1,t2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LL.R
\name{LL}
\alias{LL}
\title{likelihood function for hyperparameters estimation}
\usage{
LL(param, x, d0)
}
\arguments{
\item{param}{parameters to be determined by MLE}

\item{x}{distance matrix of cells}

\item{d0}{rate parameter of prior of 1 / true distance}
}
\value{
return hyperparameteres a.
}
\description{
likelihood function for hyperparameters estimation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractInfo.R
\name{extractInfo}
\alias{extractInfo}
\title{extract count matrix from SingleCellExperiment object}
\usage{
extractInfo(data)
}
\arguments{
\item{data}{SingleCellExperiment object}
}
\value{
list of count matrix and condition vector
}
\description{
extract count matrix from SingleCellExperiment object
}
\examples{
data(sim_dat) 
dat <- extractInfo(sim_dat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdd.R
\name{pdd}
\alias{pdd}
\title{calculate posterior probabilities of a gene to be differential distributed}
\usage{
pdd(
  data,
  cd,
  bp,
  D,
  random = TRUE,
  norm = TRUE,
  epi = 1,
  Upper = 1000,
  nrandom = 50,
  iter = 20,
  reltol = 0.001,
  stp1 = 1e-06,
  stp2 = 0.01,
  K = 0
)
}
\arguments{
\item{data}{normalized preprocessed transcripts}

\item{cd}{conditions label}

\item{bp}{bioc parallel parameter}

\item{D}{distance matrix of cells or cluster of cells or a given clustering}

\item{random}{boolean indicator of whether randomzation has been been implemented on distance matrix}

\item{norm}{boolean indicator of whether the input expression data is normalized}

\item{epi}{tol for change of validity score in determining number of clusters}

\item{Upper}{bound for hyper parameters optimization}

\item{nrandom}{number of random generated distance matrix}

\item{iter}{max number of iterations for EM}

\item{reltol}{relative tolerance for optim on weighting paramters}

\item{stp1}{step size of hyperparameter alpha (shared by all units) in one step EM}

\item{stp2}{step size of hyperparameter beta (unit specific) in one step EM}

\item{K}{number of subtypes, could be user specified or determined internally(set to 0)}
}
\value{
posterior probabilities of a gene to be differential distributed
}
\description{
calculate posterior probabilities of a gene to be differential distributed
}
\examples{
data(sim_dat)
dat <- extractInfo(sim_dat)
data_counts <- dat$count_matrix
cd <- dat$condition
bp <- BiocParallel::MulticoreParam(4)
D_c <- calD(data_counts,bp)
pDD <- pdd(data_counts,cd,bp,D_c)
}
